﻿define( ['qvangular',
		'text!QvBigQueryConnector.webroot/connectdialog.ng.html',
        'css!QvBigQueryConnector.webroot/connectdialog.css'
], function (qvangular, template) {
	return {
		template: template,
        controller: ['$scope', 'input', function ($scope, input) {

            function getParameter(parameterName, scopeMember) {
                return input.serverside.sendJsonRequest("getConnectionParameter", parameterName).then(function (info) {
                    console.log("info", info, scopeMember)
                    $scope[scopeMember] = info.qMessage;
                    return qvangular.promise(info.qMessage);
                });
            }

			function init() {
                //gui Seyup
                $scope.isEdit = input.editMode;
				$scope.id = input.instanceId;
				$scope.titleText = $scope.isEdit ? "Change BigQuery Connector" : "Add BigQuery Connector";
				$scope.saveButtonText = $scope.isEdit ? "Save changes" : "Create";

                //Bigquery + Storage variables
                $scope.cacheDataSet = "";
                $scope.cacheBucket = "";
                $scope.project = "";                
                $scope.serviceAccountName = "";
                $scope.serviceAccountKey = "";

                //Qlik Variables
                $scope.name = "";
                $scope.provider = "BigQueryConnector.exe"; // Connector filename

                //gui variables
                $scope.connectionInfo = "";
                $scope.connectionSuccessful = false;
                $scope.testingInProgress = false;
                $scope.useCSVLoader = false;

                //initial connection string
                $scope.connectionString = createCustomConnectionString($scope.provider);

                //for switching conenctor mode : default avroLoader
                $scope.connectorMode = 'avroLoader';

                //for csvloader
                $scope.serverCacheFolderPath = "";
                $scope.qlikFolderConnection = "";

                //Get info fom Connector for Debug
				input.serverside.sendJsonRequest( "getInfo" ).then( function ( info ) {
					$scope.info = info.qMessage;
				} );

                if ($scope.isEdit) {

                    //get connection name
                    input.serverside.getConnection($scope.id).then(function (result) {
                        console.log("result", result);
                        $scope.name = result.qConnection.qName;
                    });

                    //get custom parameters
                    getParameter("cacheDataSet", "cacheDataSet");
                    getParameter("project", "project");
                    getParameter("UserId", "serviceAccountName").then(function(name){
                        $scope.currentServiceAccountName = name;
                    });
                    getParameter("cacheBucket", "cacheBucket");
                    getParameter("connectorMode", "connectorMode").then(function (connectorMode) {
                        if (connectorMode == "csvLoader") {
                            $scope.useCSVLoader = true;
                        };
                    });
                    getParameter("serverCacheFolderPath", "serverCacheFolderPath");
                    getParameter("qlikFolderConnection", "qlikFolderConnection");

                }

            }

            console.log('org connect dialog', Date.now(), input);

            //functions

            function validScopeInput(scopeVariableArray) {
                for (index = 0; index < scopeVariableArray.length; ++index) {
                    scopeVariable = scopeVariableArray[index];
                    //console.log("test", scopeVariable.property, $scope[scopeVariable.property], $scope[scopeVariable.property].length, !!$scope[scopeVariable.property], $scope[scopeVariable.property].length > 0, !!scopeVariable.regex, scopeVariable.regex);
                    if (!!$scope[scopeVariable.property] && $scope[scopeVariable.property].length > 0) {
                        //has regex?
                        if (!!scopeVariable.regex) {                            
                            if (!scopeVariable.regex.test($scope[scopeVariable.property])) {
                                return { message: "invalid format " + scopeVariable.regexError + "", valid: false };
                            } 
                        }
                        //ok
                    } else {
                        return { message: "missing " + scopeVariable.error, valid: false };
                    }
                }
                return { message: "Input ok", valid: true };
            }


			/* Event handlers */

			$scope.onOKClicked = function () {
				if ( $scope.isEdit ) {
                    var overrideCredentials = ($scope.serviceAccountName !== "" && $scope.serviceAccountKey !== "");
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.cacheDataSet, $scope.cacheBucket, $scope.project, $scope.connectorMode, $scope.serverCacheFolderPath, $scope.qlikFolderConnection);
                    console.log("updating connection with name", $scope.name, "serviceaccountName", $scope.serviceAccountName, "connection string", $scope.connectionString);
                    input.serverside.modifyConnection( $scope.id,
						$scope.name,
						$scope.connectionString,
						$scope.provider,
						overrideCredentials,
						$scope.serviceAccountName,
						$scope.serviceAccountKey ).then( function ( result ) {
                            if (result) {
                                //console.log("result", result);
								$scope.destroyComponent();
							}
						} );
				} else {
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.cacheDataSet, $scope.cacheBucket, $scope.project, $scope.connectorMode, $scope.serverCacheFolderPath, $scope.qlikFolderConnection );
                    input.serverside.createNewConnection($scope.name, $scope.connectionString, $scope.serviceAccountName, $scope.serviceAccountKey);
					$scope.destroyComponent();
				}
			};

            $scope.onTestConnectionClicked = function () {

                $scope.connectionInfo = "";

                var scopeVariables = [
                    { property: "name", error: "connection name" },
                    { property: "serviceAccountName", error: "connector name", regex: /[a-zA-Z0-9_]+$/, regexError: "only a-z A-Z 0-9 and _ is allowed" },
                    { property: "project", error: "project" },
                    { property: "cacheDataSet", error: "dataset" },
                    { property: "cacheBucket", error: "bucket" },
                ];

                if (!$scope.isEdit || $scope.serviceAccountName != $scope.currentServiceAccountName ) {
                    scopeVariables.push({ property: "serviceAccountKey", error: "json key" });
                }

                if ($scope.useCSVLoader) {
                    scopeVariables.push({ property: "serverCacheFolderPath", error: "server folder path" });
                    scopeVariables.push({ property: "qlikFolderConnection", error: "folder connection name" });
                }

                var inputValidation = validScopeInput(scopeVariables);

                if (!inputValidation.valid) {
                    $scope.connectionInfo = inputValidation.message;
                    $scope.connectionSuccessful = false;
                    return;
                }


                $scope.testingInProgress = true;
                input.serverside.sendJsonRequest("testConnection", $scope.serviceAccountName, $scope.serviceAccountKey, $scope.project, $scope.cacheDataSet, $scope.cacheBucket).then(function (info) {
                    console.log("Info", info);
                    $scope.connectionInfo = info.message;
                    $scope.connectionSuccessful = info.access;
                    $scope.testingInProgress = false;
                    console.log($scope.connectionSuccessful);
				} );
			};

            $scope.isOkEnabled = function () {
                return !!$scope.name && $scope.name.length > 0 && $scope.connectionSuccessful && !$scope.testingInProgress;
			};

			$scope.onEscape = $scope.onCancelClicked = function () {
				$scope.destroyComponent();
            };

            /* Watches */

            $scope.$watch("useCSVLoader", function (newValue, oldValue) {
                if (newValue != oldValue) {

                    if (newValue) {
                        $scope.connectorMode = "csvLoader";
                    } else {
                        $scope.connectorMode = "avroLoader";
                    }
                }
            });

			
			/* Helper functions */


            function createCustomConnectionString(provider, cacheDataSet, cacheBucket, project, connectorMode, serverCacheFolderPath, qlikFolderConnection) {

                if (!!serverCacheFolderPath && serverCacheFolderPath.length > 0) {
                    serverCacheFolderPath = "serverCacheFolderPath=" + serverCacheFolderPath + ";"
                }

                if (!!qlikFolderConnection && qlikFolderConnection.length > 0) {
                    qlikFolderConnection = "qlikFolderConnection=" + qlikFolderConnection + ";"
                }

                var defaultCleanupSettings = "bucketCleanup=10;localCleanup=10;deletePolicyInhours=168;"

                var connectionString = "CUSTOM CONNECT TO " + "\"provider=" + provider + ";cacheDataSet=" + cacheDataSet + ";cacheBucket=" + cacheBucket + ";project=" + project + ";connectorMode=" + connectorMode + ";" + serverCacheFolderPath + qlikFolderConnection + defaultCleanupSettings + "\"";

                console.log("string", connectionString);
                return connectionString;
            }

            // Help Menu


			init();
		}]
	};
} );