define(['qvangular',
    'underscore',
    'text!QvBigQueryConnector.webroot/selectdialog.ng.html',
    'css!QvBigQueryConnector.webroot/selectdialogstyle.css'
], function (qvangular, _, template) {
    return {
        template: template,
        controller: ['$scope', 'serverside', function ($scope, serverside) {
            var bigQueryDialogContentProvider = {
                getConnectionInfo: function () {
                    return qvangular.promise({
                        dbusage: false,
                        ownerusage: false,
                        dbseparator: '.',
                        ownerseparator: '.',
                        specialchars: '! "$&\'()*+,-/:;<>`{}~[]',
                        quotesuffix: '"',
                        quoteprefix: '"',
                        dbfirst: true,
                        keywords: []
                    });
                },
                getConnectorMode: function () {
                    //start loading spinner
                    $scope.isLoading = true;

                    serverside.sendJsonRequest("getConnectionParameter", "connectorMode").then(function (mode) {
                        console.log("Connector is in mode: " + mode.qMessage);
                        $scope.connectorMode = mode.qMessage;

                        if ($scope.connectorMode == "csvLoader") {
                            console.log("Getting csv loader script")

                            serverside.sendJsonRequest("getCSVLoderSript").then(function (script) {
                                if (script.type == "error") {
                                    console.error("Failed to get csv script");
                                    $scope.googleQuery = script.qMessage;
                                    $scope.selectDialogState = "ERROR";
                                } else {
                                    console.log("Got csv script: " + script.qMessage);
                                    $scope.csvScript = script.qMessage;
                                }

                                //stop spinner
                                $scope.isLoading = false;
                            });
                        } else {
                            //stop spinner
                            $scope.isLoading = false;
                        }
                    });
                },

                //getBuckets: function (projectID) {
                //    $scope.loadInProgress = true;

                //    return serverside.sendJsonRequest("getBuckets", projectID).then(function (response) {
                //        $scope.loadInProgress = false;

                //        console.log("bucket response", response);
                //        if (response.type && response.type == "Error") {
                //            $scope.errorMessage = "No catalog was returned";
                //            console.log($scope.errorMessage);
                //            return [{ "qName": "No catalog was returned" }];
                //        }
                //        return response.GoogleBuckets;
                //    });
                //},

                getPreview: function ( /*databaseName, ownerName, tableName*/) {
                    return qvangular.promise([]);
                },

                getProjects: function () {

                    //return qvangular.promise([
                    //    {
                    //        name: "first project",
                    //        id: "P01"
                    //    },
                    //    {
                    //        name: "second projct",
                    //        id: "P02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getProjects").then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("projects response", response);
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No prjects was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No projects was returned" }];
                        }
                        return qvangular.promise(response.Projects);
                    });
                },

                getDatasets: function (projectID) {

                    //return qvangular.promise([
                    //    {
                    //        name: "first Dataset",
                    //        id: "D01"
                    //    },
                    //    {
                    //        name: "second Dataset",
                    //        id: "D02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getDatasets", projectID).then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("dataset response", response);
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No datasets was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No datasets was returned" }];
                        }
                        return qvangular.promise(response.Datasets);
                    });
                },
                getTables: function (projectID, dataSetID) {
                    console.log("tables - Dataset", dataSetID);

                    if (!projectID || !dataSetID) {
                        return qvangular.promise([]);
                    }

                    //return qvangular.promise([
                    //    {
                    //        name: projectID + " " + dataSetID + " first table",
                    //        id: "T01"
                    //    },
                    //    {
                    //        name: projectID + " " + dataSetID + " second table",
                    //        id: "T02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getTables", projectID, dataSetID).then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("tables response", response)
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No tables was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No tables was returned" }];
                        }
                        return qvangular.promise(response.Tables);
                    });
                },
                getFields: function (projectID, dataSetID, tableID) {
                    console.log("Field - Project", projectID, "Table", tableID, "DataSet", dataSetID );

                    //return qvangular.promise([
                    //    {
                    //        name: projectID + " " + dataSetID + " " + tableID + " first field",
                    //        id: "F01"
                    //    },
                    //    {
                    //        name: projectID + " " + dataSetID + " " + tableID + " second field",
                    //        id: "F02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getFields", projectID, dataSetID, tableID).then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("Field response", response)
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No fields was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No fields was returned" }];
                        }
                        return qvangular.promise(response.Fields);
                    });
                },

            };


            //helpers
            var DEBUG = true;

            //wrapping the logging function
            var log = function () {
                if (DEBUG) {
                    console.log.apply(console, arguments);
                }
            }

            //Filters

            //init of scope
            function init() {

                //Model variables 
                $scope.projects = [];
                $scope.dataSets = [];
                $scope.tables = [];
                $scope.fields = [];


                $scope.errorMessage = "";
                $scope.loadInProgress = false;
                $scope.googleQuery = "";
                //$scope.project = "";

                $scope.connectorMode = "";
                $scope.csvScript = "";

                $scope.selectDialogState = "OK";

                bigQueryDialogContentProvider.getConnectorMode();

                //View control variables
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;

                //GUI mode constrol
                $scope.showSelectionGUI = true;

                $scope.currentProject = null;
                $scope.currentDataSet = null;
                //$scope.currentTable = "Select table";

                //query settings
                $scope.options = [
                    {
                        name: "Use BigQuery Cache",
                        settingName: "USE_BIGQUERY_CACHE",
                        value: false
                    },
                    {
                        name: "PROCESSING_LOCATION",
                        settingName: "PROCESSING_LOCATION",
                        value: "AUTO"
                    }
                ];

                //get datasets in project
                bigQueryDialogContentProvider.getProjects().then(function (projects) {
                    console.log("projects", projects);
                    $scope.projects = projects;
                });                
            
            }


            console.log('org select dialog', Date.now(), $scope, bigQueryDialogContentProvider);


            //GUI fucntions

            $scope.getDatasets = function (projectID) {
                bigQueryDialogContentProvider.getDatasets(projectID).then(function (dataSets) {
                    console.log("getDatasets scope", dataSets);
                    $scope.dataSets = dataSets;
                    $scope.selectedTable = null;//clear table gui
                    $scope.tables = [];
                });  
            };

            $scope.getTables = function (projectID, dataSetID) {
                bigQueryDialogContentProvider.getTables(projectID, dataSetID).then(function (tables) {
                    console.log("getTables scope", tables);
                    $scope.tables = tables;
                    $scope.selectedTable.fields = null; //clear field gui
                });
            }

            function getFields(projectID, dataSetID, tableID) {
                //get table
                var table = $scope.tables.find(table => table.id === tableID);
                console.log("getFields", table, !!table.fields );
                if (!table.fields) {
                    //get fields
                    //$scope.getFields($scope.currentDataSet.id, table.id);

                    return bigQueryDialogContentProvider.getFields(projectID, dataSetID, tableID).then(function (fields) {
                        console.log("got Fields", fields);

                        //find table in scope and add fields

                        table.fields = fields; //add fields to table
                        table.numberOfSelectedFields = 0; //set number of selected
                        //$scope.fields = table.fields; //assign fieds to current fields

                        return qvangular.promise(table.fields);
                    });
                } else {
                    //$scope.fields = table.fields;
                    console.log("scope tables get fields", $scope.tables, $scope.selectedTable.fields);
   
                    return qvangular.promise(table.fields);
                }

            }

            $scope.showFieldDataLoader = function () {
                return $scope.loadInProgress || $scope.loadingPreview > 0;
            };

            $scope.isTableSelected = function (table) {
                return (table === $scope.selectedTable);
            };


            //Internal Functions


            // This accomodates the recently changed QlikView API contract
            // without having to make changes to the contract for the connect dialog.
            ///// see C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvOdbcConnectorPackage\web\selectdialog.js ///

            function indentLine(level) {
                var indent = "";
                for (var i = 0; i < level; i++) {
                    indent += "\t";
                }
                return indent;
            }

            //build query from selection
            function GetScriptFromQuery(tables, limit, project, options) {
                console.log("Build script from GUI");

                var selectedTables = tables.filter(table => table.checked);
                console.log("found selected tables", selectedTables);

                var script = "";

                //loop tables
                selectedTables.forEach(function (table) {
                    var query = "Select\r\n";
                    var queryFields = "";

                    //find selected fields
                    var seletedFields = table.fields.filter(field => field.checked);
                    console.log("selected fields", seletedFields);

                    //loop selected fields
                    seletedFields.forEach(function (field) {
                        queryFields += indentLine(1) + "`" + field.id + "`,\r\n";
                    });

                    //add query fields
                    query += queryFields.slice(0, -3) + "\r\n";

                    //add table reference
                    query += indentLine(0) + "FROM `" + table.fullyQualifiedId + "`";

                    //add limit for preview
                    if (!!limit && Number.isInteger(limit) && limit > 0) {
                        query += indentLine(1) + "Limit " + limit;
                    }

                    script += GetCurrentScript(query, project, options, table.name);
                });

                return script;

            }

            //Query wrapper
            function GetCurrentScript(googleQueryInput, project, options, tableName) {
                console.log("Generate Script", googleQueryInput, project, options);
                var script = "";
                var optionQuery = "";
                var googleQuery = "";

                //parse Options
                options.forEach(function (option) {
                    optionQuery += indentLine(2) + option.settingName + " " + option.value + ",\r\n";
                });

                //add indent to lines of query
                googleQueryInput.split(/\r?\n/).forEach(function (line) {
                    googleQuery += indentLine(2) + line + "\r\n";
                });

                //csvloder script
                if ($scope.connectorMode == "csvLoader") {
                    script += indentLine(0) + "///// START BigQuery Load /////\r\n";
                    script += indentLine(0) + "[CSVFiles]:\r\n";
                } else {
                    script += indentLine(0) + "[" + tableName + "]:\r\n";
                }

                script += indentLine(0) + "SELECT\r\n";
                script += indentLine(1) + "QUERY\r\n";
                script += indentLine(0) + googleQuery.slice(0, -2) + "\r\n";
                script += indentLine(1) + "PROJECT " + project + "\r\n";
                script += indentLine(1) + "OPTIONS\r\n";
                script += indentLine(0) + optionQuery.slice(0, -3) + ";\r\n";

                //csvloder script
                if ($scope.connectorMode == "csvLoader") {
                    script += indentLine(0) + $scope.csvScript.replace("%%BigQueryTableName%%", "[" + tableName + "]") + "\r\n";
                    script += indentLine(0) + "///// END BigQuery Load /////\r\n";
                }

                console.log("Generated Script", googleQueryInput, project, options, script);

                script += "\r\n\r\n";
                return script;
            }
            

              ///// Event handlers //////

            //tables
            $scope.onTableShowClick = function ($event) {
                console.log("onTableShowClick", $event);
                if (!$scope.editLockdown) {
                    var table = $($event.target).scope().table;
                    $scope.onTableShow(table);
                }
            };

            $scope.onTableShow = function (table) {
                if (table && table !== $scope.selectedTable) {
                    $scope.selectedTable = table;

                    getFields($scope.currentProject.id, $scope.currentDataSet.id, table.id);
                }
            };

            $scope.onTableCheckboxClick = function ($event) {
                console.log("onTableCheckboxClick", $event);
                if (!$scope.editLockdown) {
                    var targetScope = $($event.target).scope();
                    var table = targetScope.table;
                    table.proptest = 2;

                    getFields($scope.currentProject.id, $scope.currentDataSet.id, table.id).then(function (fields) {
                        //udate the table status
                        if (!table.checked) {
                            table.checked = true;
                        } else {
                            table.checked = !table.checked;
                        }

                        //set same status as the table to all fields
                        fields.forEach(function (field) {
                            field.checked = table.checked;   
                        });
                        //update date number over selected fields (could be move to a watch)
                        table.numberOfSelectedFields = fields.length;
                    });
                }
                console.log("scope tables on check table", $scope.tables);
            };

            //fields
            $scope.handleFieldClick = function ($event) {

                //console.log("event", $event);

                var targetScope = $($event.target).scope();
                var field = targetScope.field;

                console.log("handleFieldClick", field);

                if (!field.checked) {
                    field.checked = true;
                } else {
                    field.checked = !field.checked;
                }

                var checkedFields = $scope.selectedTable.fields.filter(field => field.checked).length;

                $scope.selectedTable.numberOfSelectedFields = checkedFields;

                //check the table has checked fields (not optimized for readability)
                if (checkedFields > 0) {
                    $scope.selectedTable.checked = true;                    
                } else {
                    $scope.selectedTable.checked = false;
                }
            };

            $scope.onDataPreviewClick = function () {
                $scope.dataview = true;
                $scope.helpview = false;
                $scope.metadataview = false;

                //if (!$scope.preview || $scope.preview.length === 0) {
                //    loadDataPreview();
                //}
            };

            $scope.onContentConfigClick = function () {
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;
            };


            $scope.onHelpClick = function () {
                $scope.dataview = false;
                $scope.helpview = true;
                $scope.metadataview = false;
            };


            $scope.isAddScriptButtonEnabled = function () {

                //checks for an error state in the connector
                if (!!$scope.selectDialogState && $scope.selectDialogState != "ERROR") {
                     //NOT GUI and check a query is pressent
                    if (!$scope.showSelectionGUI && !!$scope.googleQuery && $scope.googleQuery.length > 0) {
                        //check project is defined
                        if (!!$scope.currentProject.id && $scope.currentProject.id.length > 0) {
                            return true;
                        }
                    }
                    //GUI and selected are pressent 
                    if ($scope.showSelectionGUI && $scope.tables.filter(table => table.checked).length > 0) {
                        return true;
                    }
                }
                return false;
            };


            //SEE Rest connector C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvRestConnector\web\selectdialog.js
            $scope.onOK = function () {
                $scope.isLoading = true;

                if (!$scope.showSelectionGUI) {
                    console.log("Script from direct query");
                    var script = GetCurrentScript($scope.googleQuery, $scope.currentProject.id, $scope.options,"BigQuery");
                } else {
                    console.log("Script from GUI selection");
                    var script = GetScriptFromQuery($scope.tables, null, $scope.currentProject.id, $scope.options);
                }


                $scope.scriptEditorProxy.insertCode(script, $scope.dataconnectorInstance);
                $scope.destroyComponent();
            };

            $scope.onCancel = function () {
                $scope.cancelIsInProgress = true;
                $scope.destroyComponent();

            };

            //init model

            init();

        }]
    };
});



